/* 
 * File:   specAna.c
 * Author: Hardie Pienaar
 *
 * Created on 03 October 2012, 5:48 PM
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <xc.h>

#define _XTAL_FREQ 4000000

#pragma config FOSC=INTRCIO, WDTE=OFF, PWRTE=OFF, MCLRE=ON, CP=OFF, \
                CPD=OFF, BOREN=OFF, IESO=OFF, FCMEN=OFF

void delay10(char n);
void sendUARTByte(unsigned char dataValue);
void sendUARTWord(unsigned char* dataWord);
void setupUART();
int receivedUARTData();
char readUARTByte();

void setupSPI();
char readSPIByte();
void writeSPIByte(char byte);

char readRFM22BRegister(char address);
void writeRFM22BRegister(char data,char address);
void configureRadio();

void showIntroTest();
void sendUARTInByteFormat(char data);

void setRadioFrequency( double freq);
/*
Pin Connections
NSEL - RC0
NIRQ - RC1
*/

int main(int argc, char** argv) {
    
    //Initialize ports
    TRISC = 0b00000000;
    PORTC = 0b00000011;
    TRISB = 0xF0;
    PORTB = 0;

    //Setup UART and SPI Communication channels
    setupUART();
    setupSPI();
    delay10(10);
    
    sendUARTWord("\033[2J"); //Clear hyperterminal screen
    setRadioFrequency(598.434);
    /*showIntroTest();
    configureRadio();
    showIntroTest(); //Test
    sendUARTWord("RSSI: 00000000"); //Test*/

    while(1)
    {
       //Test - Echo hyperterminal values
       if(receivedUARTData())
           sendUARTByte(readUARTByte());
            
       //Test Show RSSI value
       /*sendUARTWord("\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
       sendUARTWord("RSSI: ");
       sendUARTInByteFormat(readRFM22BRegister(0x26));
       delay10(50);*/
      
    }

    return (EXIT_SUCCESS);
}

/*Creates a delay adjustable by parameter n*/
void delay10( char n)
{
    int i = 0;
    int j = 0;

    for(i = 0;i < 255;i++)
        for(j = 0;j < n;j++)
        {}
}

/*Configures the radio to default settings used for
 spectrum analyser mode*/
void configureRadio()
{
     sendUARTWord("Configuring Radio\r\n");
     //Turn on PLL to enter tune mode
     //Turn RX on to enter receive mode
     writeRFM22BRegister(0b00000110,0x07);
}

/*Sets up the radio the specified radio frequency
 240MHz -> 960MHz, frequency is specified in MHz*/
void setRadioFrequency(double freq)
{
    //First generate the neccesary setting for the specified frequency
    char bandSelect = 0b00000000;
    char fb = 0b00000000;
    short fc = 0;
    int xtalfreq = 30000;
 

    if(freq >= 480)
    {
        bandSelect |= 0b10000000;
        fb = (int)floor((freq)/(10*(xtalfreq/30000)*(1+1))) - 24;
        double temp = freq/(10*(xtalfreq/30000)*(1+1));
        fc = (short)round((temp -floor(temp) )*64000);
    }
    else
    {
        fb = (int)floor((freq)/(10*(xtalfreq/30000)*(1+0))) - 24;
        double temp = freq/(10*(xtalfreq/30000)*(1+0));       
        fc = (short)round((temp -floor(temp) )*64000);
    }
    bandSelect |= fb;
    char fc1 = (char)(fc>>8);
    char fc2 = (char)(fc&0b0000000011111111);

    sendUARTInByteFormat(bandSelect);
    sendUARTWord("\r\n");
    sendUARTInByteFormat(fc1);
    sendUARTWord("\r\n");
    sendUARTInByteFormat(fc2);
    sendUARTWord("\r\n");
 }


/*Writes relevant register data to hyper terminal screen*/
void showIntroTest()
{
    sendUARTWord("UART PIC16F690 RFM22B Spectrum Analyser \r\n\n");
    sendUARTWord("----RF22B Device Parameters----\r\n");

    sendUARTWord("Device Type:    \t");
    sendUARTInByteFormat(readRFM22BRegister(0));
    sendUARTWord("\r\n");

    sendUARTWord("Device Version: \t");
    sendUARTInByteFormat(readRFM22BRegister(1));
    sendUARTWord("\r\n");

    sendUARTWord("Device Status:   \t");
    sendUARTInByteFormat(readRFM22BRegister(2));
    sendUARTWord("\r\n");

    sendUARTWord("Op & Func Con 1: \t");
    sendUARTInByteFormat(readRFM22BRegister(7));
    sendUARTWord("\r\n");

    sendUARTWord("Op & Func Con 2:   \t");
    sendUARTInByteFormat(readRFM22BRegister(8));
    sendUARTWord("\r\n");

    sendUARTWord("Freq Offset 1:   \t");
    sendUARTInByteFormat(readRFM22BRegister(0x73));
    sendUARTWord("\r\n");

    sendUARTWord("Freq Offset 2:   \t");
    sendUARTInByteFormat(readRFM22BRegister(0x74));
    sendUARTWord("\r\n");

    sendUARTWord("Freq Band Sel:   \t");
    sendUARTInByteFormat(readRFM22BRegister(0x75));
    sendUARTWord("\r\n");

    sendUARTWord("Nom Car Freq 1:     \t");
    sendUARTInByteFormat(readRFM22BRegister(0x76));
    sendUARTWord("\r\n");

    sendUARTWord("Nom Car Freq 2:     \t");
    sendUARTInByteFormat(readRFM22BRegister(0x77));
    sendUARTWord("\r\n");

    sendUARTWord("Freq Hop Ch Sel:    \t");
    sendUARTInByteFormat(readRFM22BRegister(0x79));
    sendUARTWord("\r\n");

    sendUARTWord("Freq Hop Step Size:   \t");
    sendUARTInByteFormat(readRFM22BRegister(0x7A));
    sendUARTWord("\r\n");

    sendUARTWord("RSSI Indicator:       \t");
    sendUARTInByteFormat(readRFM22BRegister(0x26));
    sendUARTWord("\r\n");

    sendUARTWord("RSSI Clear Ch Thres:   \t");
    sendUARTInByteFormat(readRFM22BRegister(0x27));
    sendUARTWord("\r\n");
}

/*Read a specified register from the RFM22B module via
 the SPI communication channel*/
char readRFM22BRegister(char address)
{
        RC0 = 0; //Chip select pin
        writeSPIByte(address); // Send register address
        char data = readSPIByte(); // read received data
        RC0 = 1; //Chip select pin
        return data;
}

/*Read a specified register from the RFM22B module via
 the SPI communication channel*/
void writeRFM22BRegister(char data,char address)
{
        RC0 = 0; //Chip Select Pin
        address |= 0b10000000; //Append write bit to specify a write a action
        writeSPIByte(address); //Send register address to write to
        writeSPIByte(data); // Write data to address
        RC0 = 1; //Chip Select Pin
        
}

/*Writes char value in binary form to hyper terminal*/
void sendUARTInByteFormat(char data)
{
    char temp = data;
    char subtract = 128;
    int i = 0;
    for(i = 0;i < 8;i++)
    {
        if(temp - subtract >= 0)
        {
            sendUARTByte('1');
            temp = temp - subtract;
        }
        else
            sendUARTByte('0');
        subtract = subtract/2;
    }
}

/*Setup UART communication at 8 bits and 9600 baud*/
void setupUART()
{
    TXEN = 1; //Enable Transmit
    SYNC = 0; //Asyncronous transmit
    TX9 = 0;  // 8 bit transfer selected
    SPEN =1;
    BRGH = 0; //Set to 9600 baudrate
    BRG16 = 1;
    SPBRG = 25;

    CREN = 1;
    RX9 = 0;  //Receive 8 bits
    ANSELH |= 0b11110111;//Disable specificc analog channels being used for UART

}

/*Send a single Byte via UART*/
void sendUARTByte(unsigned char dataValue)
{
    while (!TXIF) ; //Wait for last transmit to finish
    TXREG = dataValue; //Start Transmission
}

/*Send a string of bytes via UART*/
void sendUARTWord(unsigned char* dataWord)
{
    int i = 0;
    while(*(dataWord + i) != '\0') //Write until line stop
    {
           // Send a single byte from specific point in string
           sendUARTByte(*(dataWord +i));
           i++; // move to next point in string
    }
}

/*Check if a data is being received*/
int receivedUARTData()
{
    if(RCIF == 1)
    {
        RCIF = 0;
        return 1;
    }
    else
        return 0;
}

/*Fetch received data from receive buffer*/
char readUARTByte()
{
    char data = RCREG;
    return data;      //read RCREG
}

/*Setup SPI Communication channel at Fosc/4*/
void setupSPI()
{
    TRISC &= 0b01111111;         // SDO output
    TRISB &= 0b10111111;          //SCK output (master)
    TRISB |= 0b00010000;          // SDI input

    ANSEL  = 0x00;                 // Disable ADC
    ANSELH = 0x00;

    /**
        *Initialise the SPI bus on the PIC
    */

    SSPSTAT = 0b01000000;       //CKE=1 Set polarity of signal as well as trigger edge
    SSPCON = 0b00100000;        //CKP=1  and  SSPEN=1
    SSPIF=0;                    //clear SPI interrupt flag
    

}

/*Write a single byte via SPI communication channel*/
void writeSPIByte(char byte)
{
    SSPEN = 1; //Enable SPI
    SSPBUF = byte; //Send Byte
    while(!BF){} //Wait for transmission
    SSPEN = 0; //Disable SPI
}

/*Read a single byte from SPI communication channel*/
char readSPIByte()
{
    char data;
    SSPEN = 1; //Enable SPI
    SSPBUF = 0xFF; //Send dummy Byte
    while(!BF){} //Wait for transmission
    if(SSPOV != 0)SSPOV = 0;
    data = SSPBUF; //Read input buffer
    SSPEN = 0; //Disable SPI
    return data;


}